local name, ns = ...
local debug = ns.debug
local Addon = _G[name]

local L = setmetatable({}, { __index = function(self, index) return index end })

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local function escape(text)
	return text:replace("|","||")
end

local function unescape(text)
	return text:replace("||", "|")
end

local function codeIsValidLua(_, code)
	local _, err = loadstring(code)
	if not err then
		return true
	else
		return err
	end
end

local function showConfig()
	AceConfigDialog:SetDefaultSize(name, 950, 600)
	AceConfigDialog:Open(name)
	-- InterfaceOptionsFrame:Hide() -- hides everything...
end

function setOption(info, value)
	local key = info[#info]
	Addon.db.profile[key] = value
end
function getOption(info)
	local key = info[#info]
	return Addon.db.profile[key]
end

local lineChoices = {}
for i = 1,30 do
	lineChoices[i] = L["Line %d"]:format(i)
end

local currentLine = 1
local function setLineText(info, value)
	local side = info[#info - 1]
	if Addon.db.profile[side .. "Plain"][currentLine] then
		Addon.db.profile[side .. "Plain"][currentLine] = value
		return
	end
	if Addon.db.profile[side .. "Templates"][currentLine] then
		Addon.db.profile[side .. "Templates"][currentLine] = nil
	end
	Addon.db.profile[side][currentLine] = unescape(value)
end

local function getLineText(info)
	local side = info[#info - 1]
	local plain = Addon.db.profile[side .. "Plain"][currentLine]
	if plain then
		return plain
	end
	local usedTemplate = Addon.db.profile[side .. "Templates"][currentLine]
	local text
	if usedTemplate then
		text = Addon:GetTemplateValue(usedTemplate)
	else
		text = Addon.db.profile[side][currentLine]
	end
	return escape(text)
end

local templateOpts = {}
local function getTemplateChoices()
	wipe(templateOpts)
	templateOpts = CopyTable(Addon:GetTemplates())
	templateOpts["0"] = L["Custom Text"]
	templateOpts["1"] = L["Plain Text"]
	return templateOpts
end

local function getCurrentSelectText(info)
	local side = info[#info - 1]
	if Addon.db.profile[side .. "Plain"][currentLine] then
		return "1"
	end
	return Addon.db.profile[side .. "Templates"][currentLine] or "0"
end

local function setTemplateText(info, value)
	local side = info[#info - 1]
	local old = Addon.db.profile[side .. "Templates"][currentLine]
	if value == "1" then
		Addon.db.profile[side .. "Plain"][currentLine] = value
		value = nil
	else
		Addon.db.profile[side .. "Plain"][currentLine] = nil
	end
	if value == "0" then
		setLineText(info, Addon:GetTemplateValue(old))
		value = nil
	end
	Addon.db.profile[side .. "Templates"][currentLine] = value
end

local options
function Addon:CreateOptions()
	if options then
		return
	end

	local textContentArgs = {
		selectText = {
		type = 'select',
			name = L["select text"],
			desc = L["Select default text"],
			values = getTemplateChoices,
			get = getCurrentSelectText,
			set = setTemplateText,
			order = 1,
		},
		customText = {
			type = 'input',
			name = L["Code"],
			desc = L["Line's lua code"],
			multiline = true,
			width = "full",
			get = getLineText,
			set = setLineText,
			--validate = codeIsValidLua,
			order = 2,
		},
	}

	options = {
		name = L["Tooltip LuaText"],
		desc = L[""],
		type = 'group',
		set = setOption,
		get = getOption,
		args = {
			general = {
				type = 'group',
				name = L["General"],
				desc = L["General"],
				order = 1,
				args = {
					alwaysShowLefts = {
						type = 'toggle',
						name = L["Always show texts"],
						desc = L["Always show left texts, even if the right text is empty"],
						order = 1,
					},
					printErrors = {
						type = 'toggle',
						name = L["Show Errors"],
						desc = L["Print errors to Chatframe"],
						order = 3,
					},
				},
			},
			profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
			texts = {
				type = 'group',
				name = L["Texts"],
				desc = L["Edit Text-codes"],
				args = {
					lineselect = {
						type = 'select',
						name = L["Select line"],
						desc = L["Select the line you want to edit"],
						order = 1,
						values = lineChoices,
						get = function() return currentLine end,
						set = function(info, value) currentLine = value end,
					},
					lefts = {
						type = 'group',
						inline = true,
						name = L["Left"],
						desc = L["Edit Line Text"],
						args = textContentArgs,
					},
					rights = {
						type = 'group',
						inline = true,
						name = L["Right"],
						desc = L["Edit Line Text"],
						args = textContentArgs,
					},
				},
				order = 2,
			}
		},
	}
	local dbp = self.db.profile

	AceConfig:RegisterOptionsTable(name, options)

	local options2 = {
		name = L["Tooltip LuaText"],
		desc = L[""],
		type = 'group',
		handler = self,
		args = {
			showConfig = {
				name = L["Open Configuration"],
				desc = L["Open standalone Configuration"],
				type = 'execute',
				func = "DelayedShowConfig",
			},
		},
	}

	AceConfig:RegisterOptionsTable("Tooltip_LuaText_bliz", options2)
	AceConfigDialog:AddToBlizOptions("Tooltip_LuaText_bliz", "Tooltip LuaText")

	self:RegisterChatCommand("ttl", showConfig)
	self:RegisterChatCommand("tooltiplua", showConfig)
end

function Addon:DelayedShowConfig()
	self:ScheduleTimer(showConfig, 0.1)
	InterfaceOptionsFrame:Hide()
end
